/*
 * main.c - file operation application
 *
 * Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Application Name     -   File operations
 * Application Overview -   This is a sample application demonstrating the
 *                          file-operation capabilities of CC3100 device.
 *                          Serial-flash is used as the storage medium
 * Application Details  -   http://processors.wiki.ti.com/index.php/CC31xx_File_Operations_Application
 *                          doc\examples\file_operations.pdf
 ******************************************************************************/
#include "CC3100/include/simplelink.h"
#include "CC3100/source/protocol.h"
#include "CC3100/sl_common.h"
#include <stdio.h>

/****Program controls***********************************************************
 * This program can perform three functions:
 *      (1) The first is the TI Old MacDonald read, verify, and delete example.
 *          To use this function, set FLASH_WRITE and FLASH_DELETE to 1, and only
 *          define OLD_MACDONALD.
 *      (2) The second is to load the main.html file. Define only FLASH_WRITE and
 *          MAIN_HTML. If FLASH_DLETE is defined the file will be deleted.
 *      (3) The third is to load the led_blink.html file. Define FLASH_WRITE and
 *          FLASH_DELETE depending on the operation desired, and then define only
 *          LED_BLINK_HTML.
 * This function can only take one file as an input.
 ******************************************************************************/
#define FLASH_WRITE
#define FLASH_DELETE

#define OLD_MACDONALD
//#define MAIN_HTML
//#define LED_BLINK_HTML


#ifdef OLD_MACDONALD
extern const UINT8          oldMac[];               //extern ref to data. Data is in seperate c file
#define SL_FILE_ARRAY_NAME  oldMac                  //label for file array
#define SL_FILE_NAME        "OldMac.txt"            //FIle name to use in flash storage
#endif

#ifdef MAIN_HTML
extern const UINT8          MainHTML[];             //extern ref to data. Data is in seperate c file
#define SL_FILE_ARRAY_NAME  MainHTML                //label for file array
#define SL_FILE_NAME        "www/main.html"         //FIle name to use in flash storage
#endif

#ifdef LED_BLINK_HTML
extern const UINT8          LedHTML[];              //extern ref to data. Data is in seperate c file
#define SL_FILE_ARRAY_NAME  LedHTML                 //label for file array
#define SL_FILE_NAME        "www/blink_led.html"    //FIle name to use in flash storage
#endif

/****DEFINES*******************************************************************/
#define APPLICATION_VERSION "1.2.0"

#define SL_STOP_TIMEOUT 0xFF

#define BUF_SIZE 3120       //make sure null char ends array, and buffer is big enough to hold array
#define SIZE_64K 65536

#define	SW0_IO PORTGbits.RG15

/****Application specific status/error codes***********************************/
typedef enum
{
    DEVICE_NOT_IN_STATION_MODE = -0x7D0,        //Choosing this number to avoid overlap with host-driver's error codes
    STATUS_CODE_MAX = -0xBB8
}e_AppStatusCodes;

/****GLOBAL VARIABLES**********************************************************/
UINT8  Buff[110];                //For sprintf
UINT32 g_Status = 0;

struct
{
    UINT8 g_Buf[BUF_SIZE];
    UINT16 Len;
} uNvmemBuf;

/****STATIC FUNCTION DEFINITIONS***********************************************/
static INT32 initializeAppVariables();
static void displayBanner();

/***Application's entry point***************************************************
 * brief This example code is to demonstrate NVMEM capabilities of the device.
 *       The procedure includes the optional steps:
 *          Write:
 *              1) open/Create a user file for writing
 *              2) close the user file
 *              3) open the user file for reading
 *              4) read the data and compare with the stored buffer
 *              5) close the user file
 *          Delete:
 *              6) Delete the file
 * param None
 * return -1 or -2 in case of error, 0 otherwise. 
 ******************************************************************************/
int main(int argc, char** argv)
{
    INT32 retVal = -1;
    INT32 fileHandle = -1;
    UINT32 Token = 0;

    DDPCONbits.JTAGEN = 0;          // Disable Jtag

    SYSTEMConfig(80000000ul, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);

    INTEnableSystemMultiVectoredInt();

    //initialize the system-clock of the MCU
    initClk();

    /* Configure command line interface */
    CLI_Configure((80000000ul/(1 << OSCCONbits.PBDIV)), 921600ul);

    displayBanner();

    while(SW0_IO);

    retVal = configureSimpleLinkToDefaultState();
    if(retVal < 0)
    {
        if (DEVICE_NOT_IN_STATION_MODE == retVal)
            CLI_Write("Failed to configure the device in its default state\r\n");

        LOOP_FOREVER();
    }
    retVal = initializeAppVariables();
    ASSERT_ON_ERROR(retVal);
    CLI_Write("Device is configured in default state\r\n");

    retVal = sl_Start(0,0);
    if ((retVal < 0) ||
        (ROLE_STA != retVal) )
    {
        CLI_Write("Failed to start the device\r\n");
        LOOP_FOREVER();
    }
    CLI_Write("Device started as STATION\r\n");

#ifdef FLASH_WRITE

    //Open a user file for writing
    retVal = sl_FsOpen(SL_FILE_NAME,FS_MODE_OPEN_WRITE, &Token, &fileHandle);
    if(retVal < 0)
    {
        //If file does not exist create a new one of 63 KB file
        retVal = sl_FsOpen(SL_FILE_NAME,
                           FS_MODE_OPEN_CREATE(SIZE_64K,_FS_FILE_OPEN_FLAG_COMMIT|_FS_FILE_PUBLIC_WRITE),
                           &Token, &fileHandle);
        if(retVal < 0)
        {
            CLI_Write("Error in creating the file\r\n");
            LOOP_FOREVER();
        }
    }

    //Write file to CC3100 flash memory
    retVal = sl_FsWrite(fileHandle, 0, (UINT8 *)SL_FILE_ARRAY_NAME, uNvmemBuf.Len);
    if (retVal < 0)
    {
       CLI_Write("Error in writing the file\r\n");

       retVal = sl_FsClose(fileHandle, 0, 0, 0);
       retVal = sl_FsDel(SL_FILE_NAME, Token);

       LOOP_FOREVER();
    }
       
    //Close the user file
    retVal = sl_FsClose(fileHandle, 0, 0, 0);
    if (retVal < 0)
    {
        CLI_Write("Error in closing the file\r\n");
        retVal = sl_FsDel(SL_FILE_NAME, Token);

        LOOP_FOREVER();
    }

    sprintf(Buff,"The device has written %u bytes to \"%s\"\r\nwhich is stored in the CC3100 NVM\r\n",uNvmemBuf.Len, SL_FILE_NAME);
    CLI_Write(Buff);

    //Open a user file for reading
    retVal = sl_FsOpen(SL_FILE_NAME,
                       FS_MODE_OPEN_READ, &Token, &fileHandle);
    if (retVal < 0)
    {
        CLI_Write("Error in opening the file for reading\r\n");
        retVal = sl_FsDel(SL_FILE_NAME, Token);

        LOOP_FOREVER();
    }

    //Read the data back from NVM
    memset(&uNvmemBuf.g_Buf[0], '\0', BUF_SIZE);
    retVal = sl_FsRead(fileHandle, 0, uNvmemBuf.g_Buf, uNvmemBuf.Len);
    if ( (retVal < 0) || (retVal != uNvmemBuf.Len) )
    {
        CLI_Write("Error in reading the file\r\n");
        retVal = sl_FsClose(fileHandle, 0, 0, 0);
        retVal = sl_FsDel(SL_FILE_NAME, Token);

        LOOP_FOREVER();
    }

    //Compare read data with the original file
    retVal = memcmp(SL_FILE_ARRAY_NAME, uNvmemBuf.g_Buf, uNvmemBuf.Len);
    if (retVal != 0)
    {
        CLI_Write("Contents that were written and read back didn't match\r\n");
        LOOP_FOREVER();
    }

    //Close the user file
    retVal = sl_FsClose(fileHandle, 0, 0, 0);
    if (retVal < 0)
    {
        CLI_Write("Error in closing the file\r\n");
        retVal = sl_FsDel(SL_FILE_NAME, Token);

        LOOP_FOREVER();
    }
        
    CLI_Write("Device successfully verified the following data on serial-flash\r\n\r\n");
    CLI_Write(uNvmemBuf.g_Buf);
    CLI_Write("\n\r");

#endif

#ifdef FLASH_DELETE
    //Delete the user file
    retVal = sl_FsDel(SL_FILE_NAME, Token);
    if(retVal < 0)
    {
        CLI_Write("Error in deleting the file\r\n");
        LOOP_FOREVER();
    }

    CLI_Write("Device successfully deleted the file in CC3100 NVM\r\n");

#endif

    //Stop the CC3100 device
    retVal = sl_Stop(SL_STOP_TIMEOUT);
    if(retVal < 0)
    {
        CLI_Write("Error in stopping the device\r\n");
        LOOP_FOREVER();
    }

    CLI_Write("Finished\r\n");

    return 0;
}

/****ASYNCHRONOUS EVENT HANDLERS***********************************************/

/*******************************************************************************
 * brief This function handles WLAN events
 * param[in] pWlanEvents is the event passed to the handler
 * return none
 ******************************************************************************/
void SimpleLinkWlanEventHandler(SlWlanEvent_t *pWlanEvent)
{
    if(pWlanEvent == NULL)
    {
        CLI_Write("[WLAN EVENT] NULL Pointer Error\r\n");
        return;
    }

    switch(pWlanEvent->Event)
    {
        case SL_WLAN_CONNECT_EVENT:
        {
            SET_STATUS_BIT(g_Status, STATUS_BIT_CONNECTION);

            /*
             * Information about the connected AP (like name, MAC etc) will be
             * available in 'slWlanConnectAsyncResponse_t' - Applications
             * can use it if required
             *
             * slWlanConnectAsyncResponse_t *pEventData = NULL;
             * pEventData = &pWlanEvent->EventData.STAandP2PModeWlanConnected;
             *
             */
        }
        break;

        case SL_WLAN_DISCONNECT_EVENT:
        {
            slWlanConnectAsyncResponse_t*  pEventData = NULL;

            CLR_STATUS_BIT(g_Status, STATUS_BIT_CONNECTION);
            CLR_STATUS_BIT(g_Status, STATUS_BIT_IP_ACQUIRED);

            pEventData = &pWlanEvent->EventData.STAandP2PModeDisconnected;

            /* If the user has initiated 'Disconnect' request, 'reason_code' is SL_USER_INITIATED_DISCONNECTION */
            if(SL_USER_INITIATED_DISCONNECTION == pEventData->reason_code)
            {
                CLI_Write("Device disconnected from the AP on application's request\r\n");
            }
            else
            {
                CLI_Write("Device disconnected from the AP on an ERROR!\r\n");
            }
        }
        break;

        default:
        {
            CLI_Write("[WLAN EVENT] Unexpected event\r\n");
        }
        break;
    }
}

/*******************************************************************************
 * brief This function handles events for IP address acquisition via DHCP indication
 * param[in] pNetAppEvent is the event passed to the handler
 * return None
 ******************************************************************************/
void SimpleLinkNetAppEventHandler(SlNetAppEvent_t *pNetAppEvent)
{
    if(pNetAppEvent == NULL)
    {
        CLI_Write("[NETAPP EVENT] NULL Pointer Error\r\n");
        return;
    }

    switch(pNetAppEvent->Event)
    {
        case SL_NETAPP_IPV4_IPACQUIRED_EVENT:
        {
            SET_STATUS_BIT(g_Status, STATUS_BIT_IP_ACQUIRED);

            /*
             * Information about the connected AP's IP, gateway, DNS etc
             * will be available in 'SlIpV4AcquiredAsync_t' - Applications
             * can use it if required
             *
             * SlIpV4AcquiredAsync_t *pEventData = NULL;
             * pEventData = &pNetAppEvent->EventData.ipAcquiredV4;
             * <gateway_ip> = pEventData->gateway;
             *
             */
        }
        break;

        default:
        {
            CLI_Write("[NETAPP EVENT] Unexpected event\r\n");
        }
        break;
    }
}

/*******************************************************************************
 * brief This function handles callback for the HTTP server events
 * param[in] pHttpEvent - Contains the relevant event information
 * param[in] pHttpResponse - Should be filled by the user with the
 *           relevant response information
 * return None
 ******************************************************************************/
void SimpleLinkHttpServerCallback(SlHttpServerEvent_t *pHttpEvent,
                                  SlHttpServerResponse_t *pHttpResponse)
{
    /*
     * This application doesn't work with HTTP server - Hence these
     * events are not handled here
     */
    CLI_Write("[HTTP EVENT] Unexpected event\r\n");
}

/*******************************************************************************
 * brief This function handles general error events indication
 * param[in] pDevEvent is the event passed to the handler
 * return None
 ******************************************************************************/
void SimpleLinkGeneralEventHandler(SlDeviceEvent_t *pDevEvent)
{
    /*
     * Most of the general errors are not FATAL are are to be handled
     * appropriately by the application
     */
    CLI_Write("[GENERAL EVENT]\r\n");
}

/*******************************************************************************
 * brief This function handles socket events indication
 * param[in] pSock is the event passed to the handler
 * return None
 ******************************************************************************/
void SimpleLinkSockEventHandler(SlSockEvent_t *pSock)
{
    if(pSock == NULL)
    {
        CLI_Write("[SOCK EVENT] NULL Pointer Error\r\n");
        return;
    }

    switch( pSock->Event )
    {
        case SL_SOCKET_TX_FAILED_EVENT:
        {
            /*
            * TX Failed
            *
            * Information about the socket descriptor and status will be
            * available in 'SlSockEventData_t' - Applications can use it if
            * required
            *
            * SlSockEventData_u *pEventData = NULL;
            * pEventData = & pSock->socketAsyncEvent;
            */
            switch( pSock->socketAsyncEvent.SockTxFailData.status )
            {
                case SL_ECLOSE:
                    CLI_Write("[SOCK EVENT] Close socket operation failed to transmit all queued packets\r\n");
                break;


                default:
                    CLI_Write("[SOCK EVENT] Unexpected event\r\n");
                break;
            }
        }
        break;

        default:
            CLI_Write("[SOCK EVENT] Unexpected event\r\n");
        break;
    }
}

/****Applications**************************************************************/

/*******************************************************************************
 * brief This function initializes the application variables
 * param[in] None
 * return 0 on success, negative error-code on error
 ******************************************************************************/
static INT32 initializeAppVariables()
{
    g_Status = 0;

    memset(&uNvmemBuf, 0, BUF_SIZE);
    uNvmemBuf.Len = strlen(SL_FILE_ARRAY_NAME);

    return SUCCESS;
}

/*******************************************************************************
 * brief This function displays the application's banner
 * param None
 * return None
 ******************************************************************************/
static void displayBanner()
{
    CLI_Write("Flash File Change - Version ");
    CLI_Write(APPLICATION_VERSION);
    CLI_Write("**************************************************\r\n");
}
